#include "cmd.h"
#include "hardw.h"
#include "vars.h"
#include "fnn.h"

#define CE F_4
#define WRI F_11

/* -------------------28F256A routines----------------------- */

#define CM_SET_UP_PRG 0x40
#define CM_SET_UP_ERASE 0x20
#define CM_ERASE 0x20
#define CM_ERASE_VER 0xA0
#define CM_PRG_VER 0xC0
#define CM_READ 0
#define CM_READ_INT 0x90
#define CM_RESET 0xFF

#define WrCmd(cmd) PPA2[KB] = cmd;\
                   CE = 0;\
                   WRI = 0;\
                   WRI = 1;\
                   CE = 1

char ch28F256A_init(char c)
{
  msg.cmd.voltage = 120;
  init_setting();
  SetF(F_14, 0); /* Upp2 = 0 */
  if (c)
  {
    set_Upp2(Upp);
    delay1ms(1);
  }
}

char ch28F256A_set_addr(char c)
{
  set_address(0);
  if (msg.cmd.address.lc.LO_HI_CHAR & 0x80) SetF(F_13, 1);
  else SetF(F_13, 0);
  if (msg.cmd.address.lc.HI_LO_CHAR & 0x01) SetF(F_16, 1);
  else SetF(F_16, 0);
  return(1);
}

char ch28F256A_read(char c)
{
  PPA2[KB] = 0xff;
  CE = 0;
  SetF(F_12, 0);
  c = PPA2[KA];
  SetF(F_12, 1);
  CE = 1;
  return(c);
}

char ch28F256A_write(char c)
{
  if (! Switch_Off)
  {
    get_chip_data();
    for (; dat.count; dat.count--)
    {
      WrCmd(CM_SET_UP_PRG);
      WrCmd(c);
      delay10us(1);
      WrCmd(CM_PRG_VER);
      /* delay10us(1); Must be 6us delay, assume it occurs naturally */
      WrCmd(CM_READ);
      if (ch28F256A_read(1) == c) return(1);
    }
    msg.ans.ans = anError;
    msg.ans.err = erWrite;
  }
  else
  {
    msg.ans.ans = anError;
    msg.ans.err = erSwOff;
  }
  return(0);
}

char ch28F256A_erase(char c)
{
  /*int cnt;*/

  if (c > eUErase) return(1);
  ch28F256A_init(1);
  set_Upp2(Upp);
  delay1ms(1);
  if (!c)
  {
    for (msg.cmd.address.l = 0;
         msg.cmd.address.l < msg.cmd.length;
         msg.cmd.address.l++)
    {
      ch28F256A_set_addr(0);
      if (! ch28F256A_write(0)) return(0);
    }
  }
  msg.cmd.address.l = 0;
  ch28F256A_set_addr(0);
  cnt = 6000;
  while(1)
  {
    if (Switch_Off)
    {
      msg.ans.ans = anError;
      msg.ans.err = erSwOff;
      return(0);
    }
    WrCmd(CM_SET_UP_ERASE);
    WrCmd(CM_ERASE);
    delay1ms(10);
    WrCmd(CM_ERASE_VER);
    /* delay10us(1); Must be 6us delay, assume it occurs naturally */
    WrCmd(CM_READ);
    while (ch28F256A_read(0) == 0xff)
    {
      cnt = 6000;
      if (msg.cmd.address.l >= msg.cmd.length) return(1);
      msg.cmd.address.l++;
      ch28F256A_set_addr(0);
    };
    if (! cnt--)
    {
      msg.ans.ans = anError;
      msg.ans.err = erErase;
      return(0);
    };
  }
}