#include "cmd.h"
#include "hardw.h"

#define CE F_11

/* -------------------2051 routines----------------------- */

char ch2051_init(char c)
{
  msg.cmd.voltage = 120;              /* Upp = 12.0V */
  init_setting();                     /* Call common init procedure */
  set_Upp1(Upp_0);                    /* RST = 0 */
  PPA1[KB] = 0;                       /* XTAL1 = 0 */
  delay1ms(10);                       /* Wait for 10 ms */
  set_Upp1(Upp_5V);                   /* RST = High */
  SetF(F_16, 1);                      /* P3.2 = High */
  CE = 1;                             /* P3.4 = High */
  PPA1[KA] = 4;                       /* P3.3 = Low; P3.5 = P3.7 = High */
  F_4 = 1;                            /* BUSY/READY = 1 */
}

char ch2051_set_addr(char c)          /* Address is set internally */
{
  if (c)                              /* Conditional write not allowed */
  {
    msg.ans.ans = anError;
    msg.ans.err = erMode;
    return(0);
  }
  return(1);
}

char ch2051_read(char c)
{
  PPA2[KB] = 0xff;                    /* Set port to Read mode */
  CE = 0;                             /* P3.4 = Low */
  c = PPA2[KA];                       /* Read data */
  CE = 1;                             /* P3.4 = High */
  PPA1[KB] = 1;                       /* XTAL1 = 1 */
  PPA1[KB] = 0;                       /* XTAL1 = 0: Increment counter */
  return(c);
}

char ch2051_write(char c)
{
  if (! Switch_Off)
  {
    PPA2[KB] = c;                     /* Apply data to P1 */
    set_Upp1(Upp);                    /* RST = 12V */
    delay10us(1);                     /* Wait for 10 us */
    SetF(F_16, 0);                    /* P3.2 = Low */
    delay10us(1);                     /* Wait for 10 us */
    SetF(F_16, 1);                    /* P3.2 = High */
    delay10us(1);                     /* Wait for 10 us */
    PPA2[KB] = 0xff;                  /* Set port to Read mode */
    set_Upp1(Upp_5V);                 /* RST = High */
    for (c = 128; (! F_4) && (c != 0); c--)
      delay10us(2);                   /* Wait while BUSY */
    if (F_4 && (ch2051_read(1) == c))
      return(1);                      /* OK */
    msg.ans.ans = anError;
    msg.ans.err = erWrite;
  }
  else
  {
    msg.ans.ans = anError;
    msg.ans.err = erSwOff;
  }
  return(0);
}

char ch2051_erase(char c)
{
  ch2051_init(0);
  if (! Switch_Off)
  {
    if ((c == eErase) || (c == eUErase))
    {
      PPA1[KA] = 2;                   /* P3.3 = High; P3.5 = P3.7 = Low */
      CE = 0;                         /* P3.4 = Low */
      set_Upp1(Upp);                  /* RST = High */
      SetF(F_16, 0);                  /* P3.2 = Low */
      delay1ms(10);                   /* Wait for 10 ms */
    }
    else if (c == eLockRead)
    {
      PPA1[KA] = 6;                   /* P3.3 = High; P3.5 = P3.7 = High */
      CE = 1;                         /* P3.4 = High */
      set_Upp1(Upp);                  /* RST = High */
      SetF(F_16, 0);                  /* P3.2 = Low */
      delay10us(1);                   /* Wait for 10 us */
    }
    else if (c == eLockWrite)
    {
      PPA1[KA] = 2;                   /* P3.3 = High; P3.5 = P3.7 = Low */
      CE = 1;                         /* P3.4 = High */
      set_Upp1(Upp);                  /* RST = High */
      SetF(F_16, 0);                  /* P3.2 = Low */
      delay10us(1);                   /* Wait for 10 us */
    }
    SetF(F_16, 1);                    /* P3.2 = High */
    set_Upp1(Upp_5V);
    return(1);
  }
  else
  {
    msg.ans.ans = anError;
    msg.ans.err = erSwOff;
  }
  return(0);
}