#include "cmd.h"
#include "hardw.h"

#define PROG F_11
char usr_chip_init(char c);
char usr_chip_set_addr(char c);
char usr_chip_read(char c);
char usr_chip_erase(char c);
char usr_chip_write(char c);

#define PROG F_11

/* -------------------8951 routines----------------------- */

char usr_chip_init(char c)
{
  msg.cmd.voltage = 120;              /* Upp = 12.0V */
  init_setting();                     /* Call common init procedure */
  set_Upp2(Upp_5V);                   /* Upp = 5V */
  SetF(F_16, 1);                      /* P2.7 = High */
}

char usr_chip_set_addr(char c)
{
  PPA1[KA] = msg.cmd.address.lc.LO_LO_CHAR;   /* Set address low byte */
  c = msg.cmd.address.lc.LO_HI_CHAR & 0x0F;   /* Get four lower bits */
  PPA1[KB] = (PPA1[KB] & 0xF0) | c;           /* Set four bits */
  if (msg.cmd.address.lc.LO_HI_CHAR & 0x10)
    set_Upp0(Upp_5V);                         /* Set or clear address bit */
  else set_Upp0(Upp_0);
  if (msg.cmd.address.lc.LO_HI_CHAR & 0x20)
    set_Upp1(Upp_5V);                         /* Set or clear address bit */
  else set_Upp1(Upp_0);
  if (msg.cmd.address.lc.LO_HI_CHAR & 0x40)
    SetF(F_15, 1);                            /* Set or clear address bit */
  else SetF(F_15, 0);
  return(1);
}

char usr_chip_read(char c)
{
  PPA2[KB] = 0xFF;                    /* Enable read mode */
  PPA1[KB] = (PPA1[KB] & 0xEF) | 0x60; /* P2.6 = Low, P3.6 = P3.7 = High */
  SetF(F_16, 0);                      /* P2.7 = Low */
  delay10us(2);                       /* Wait for 20 us */
  c = PPA2[KA];                       /* Read data */
  SetF(F_16, 1);                      /* P2.7 = High */
  delay10us(2);                       /* Wait for 20 us */
  return(c);
}

char usr_chip_write(char c)
{
  if (! Switch_Off)
  {
    PPA1[KB] = (PPA1[KB] & 0xEF) | 0x60; /* P2.6 = Low, P3.6 = P3.7 = High */
    SetF(F_16, 1);                       /* P2.7 = High */
    delay10us(1);                        /* Wait for 10 us */
    set_Upp2(Upp);                       /* Upp = 12V */
    PPA2[KB] = c;                        /* Apply data to P0 */
    delay10us(1);                        /* Wait for 10 us */
    PROG = 0;                            /* PROG = Low */
    delay10us(2);                        /* Wait for 20 us */
    PROG = 1;                            /* PROG = High */
    delay10us(1);                        /* Wait for 10 us */
    set_Upp2(Upp_5V);                    /* Upp = 5V */
    delay10us(1);                        /* Wait for 10 us */
    for (c = 128; (! F_4) && (c != 0); c--)
      delay10us(2);                      /* Wait while BUSY */
    if (F_4) return(1);                  /* OK */
    msg.ans.ans = anError;
    msg.ans.err = erWrite;
  }
  else
  {
    msg.ans.ans = anError;
    msg.ans.err = erSwOff;
  }
  return(0);
}

char usr_chip_erase(char c)
{
  if (c > eLockEx) return(1);
  usr_chip_init(0);
  if (! Switch_Off)
  {
    if ((c == eErase) || (c == eUErase))
    {
      PPA1[KB] = 0x10;                /* P2.6 = High, P3.6 = P3.7 = Low */
      SetF(F_16, 0);                  /* P2.7 = Low */
    }
    else if (c == eLockRead)
    {
      PPA1[KB] = 0x70;                /* P2.6 = P3.6 = P3.7 = High */
      SetF(F_16, 1);                  /* P2.7 = High */
    }
    else if (c == eLockWrite)
    {
      PPA1[KB] = 0x10;                /* P2.6 = High, P3.6 = P3.7 = Low */
      SetF(F_16, 1);                  /* P2.7 = High */
    }
    else if (c == eLockEx)
    {
      PPA1[KB] = 0x30;                /* P2.6 = High, P3.6 = High, P3.7 = Low */
      SetF(F_16, 0);                  /* P2.7 = Low */
    }
    set_Upp2(Upp);                    /* Upp = 12V */
    delay10us(2);                     /* Wait for 20 us */
    PROG = 0;                         /* PROG = Low */
    delay1ms(10);                     /* Wait for 10 ms */
    PROG = 1;                         /* PROG = High */
    delay10us(1);                     /* Wait for 10 us */
    set_Upp2(Upp_5V);                 /* Upp = 5V */
    SetF(F_16, 1);                    /* P2.7 = High */
    return(1);
  }
  else
  {
    msg.ans.ans = anError;
    msg.ans.err = erSwOff;
  }
  return(0);
}