#include "xfer.h"
#include "cmd.h"
#include "prgprocs.h"
#include "fnn.h"
#include "hardw.h"

#define write_check1(old, new) (!((old ^ new) & new))
#define write_check0(old, new) (!((old ^ new) & old))

char user(char c)
{
 return(c);
}

char mode_error(char c)
{
 msg.ans.ans = anError;
 msg.ans.err = erMode;
 return(0);
}

char ch_write(char c)
{
  char b;

  if (! chip_set_addr(1)) return(0);     /* Conditional write not allowed */
  b = chip_read(0);
  if (msg.ans.ans |= anError)
   {
    if (c != b)
     {
      if (msg.cmd.info & 0x40) b = 1;
      else if (msg.cmd.info & 0x80) b = write_check1(b, c);
           else b = write_check0(b, c);
      if (b)
       {
        if (msg.cmd.mode == mWrite)
         {
          if (chip_write(c) &&
              (c == chip_read(1)) &&
              (msg.ans.ans |= anError))
           {}
          else
           {
            if (msg.ans.ans |= anError) msg.ans.err = erWrite;
            msg.ans.ans = anError;
            return(0);
           }
         }
       }
      else
       {
        msg.ans.err = erNoWrite;
        msg.ans.ans = anError;
        return(0);
       }
     }
   }
  else
   {
    msg.ans.err = erRead;
    return(0);
   }
  return(1);
}

char read_byte(void)
{
 chip_set_addr(0);
 return (chip_read(0));
}

void Run(void);

char write_byte(char b)
{
 if (msg.cmd.mode == mLoad)
  {
   RAM[msg.cmd.address.l] = b;
   if (RRAM[msg.cmd.address.l] != b)
   {
      msg.ans.ans = anError;
      msg.ans.err = erBadAddr;
      return(0);
    }
  }
 else
  if ((msg.cmd.mode == mWrite) ||
     (msg.cmd.mode == mWrite_verify))
  {
   return(ch_write(b));
  }
 else
 if (msg.cmd.mode == mVerify)
  {
    if (b != read_byte())
     {
      msg.ans.ans = anError;
      msg.ans.err = erVerify;
      return(0);
     }
  }
 else
 if (msg.cmd.mode == mUWrite)
 {
   chip_set_addr(0);
   return(chip_write(b));
 }
 return(1);
}

void set_write(void)
{
  if ((msg.cmd.mode == mWrite) || (msg.cmd.mode == mWrite))
    chip_init(1);
  else if (msg.cmd.mode != mLoad) chip_init(0);
}

void set_read(void)
{
  chip_init(0);
}

char pWrite(void)
{
  char buf[BUF_SIZE];
  signed char cnt, i;
  char proceed;

  set_write();
  for(proceed = 1;proceed;)
    {
      if ((cnt = receive_pkt(buf, sizeof(buf))) > 0)
        {
          for(i = 0; (cnt > 0) && (proceed); cnt--)
           {
            if (proceed = write_byte(buf[i++]))
              msg.cmd.address.l ++;
           }
          if (proceed)
           {
            buf[0] = anOk;
            send_pkt(buf, 1, 0);
           }
        }
      else
        {
          if (cnt == 0)
           {
            msg.ans.ans = anOk;
            return(1);
           }
          else
           {
            msg.ans.ans = anError;
            msg.ans.err = erNoHandshake;
            return(0);
           }
        };
    }
  return(0);
}

char pRead(void)
{
  char buf[BUF_SIZE];
  signed char i;
  char proceed;

  set_read();
  for(proceed = 1; proceed;)
    {
      for(i = 0; (proceed = msg.cmd.address.l < msg.cmd.length) &&
                 (msg.ans.ans != anError) &&
                 (i < BUF_SIZE); msg.cmd.address.l ++)
       {
        if (msg.cmd.mode == mReadRAM)
          buf[i++] =  RRAM[msg.cmd.address.l];
        else buf[i++] = read_byte();
       }
      send_pkt(buf, i, 0);
      i = receive_pkt(buf, sizeof(buf));
      if ((i < 0) || (buf[0] != anOk) || (msg.ans.ans == anError))
       {
        if (i != 0) send_pkt(buf, 0, 0);
        msg.ans.ans = anError;
        return(0);
       }
    }
  if (i != 0) send_pkt(buf, 0, 0);
  return(1);
}

char pErase_Verify(void)
{
  char b;
  if (!(msg.cmd.info & 0x40))
   {
    if (msg.cmd.info & 0x80) b = 0xff;
    else b = 0;
    for(; (msg.cmd.address.l < msg.cmd.length) &&
          (msg.ans.ans != anError); msg.cmd.address.l ++)
      {
       chip_set_addr(0);
       if (b != chip_read(0))
        {
          msg.ans.ans = anError;
          msg.ans.err = erVerify;
          return(0);
        }
      }
   }
  return(1);
}

char pErase(void)
{
  return(chip_erase(eErase));
}

char pUErase(void)
{
  return(chip_erase(eUErase));
}

char pSet_Upr(char c)
{
  if (c && Switch_Off)
  {
    msg.ans.ans = anError;
    msg.ans.err = erSwOff;
    return(0);
  }
  if (c) set_Upp0(Upp);
  else set_Upp0(Upp_5V);
  if (c) set_Upp1(Upp);
  else set_Upp1(Upp_5V);
  if (c) set_Upp2(Upp);
  else set_Upp2(Upp_5V);
  return(1);
}

char pLock_Read(void)
{
  return(chip_erase(eLockRead));
}

char pLock_Write(void)
{
  return(chip_erase(eLockWrite));
}

char pLock_Ex(void)
{
  return(chip_erase(eLockEx));
}